@@ -1,4 +1,4 @@
-# Copyright (C) 2009 pdfcrowd.com
+# Copyright (C) 2009-2013 pdfcrowd.com
 # 
 # Portions of this code:
 #   <http://code.activestate.com/recipes/146306/>
@@ -25,11 +25,12 @@
 # OTHER DEALINGS IN THE SOFTWARE.
 
 import urllib
-import urllib2
 import httplib
 import mimetypes
 import socket
 
+__version__ = "2.4"
+
 
 # constants for Client.setPageLayout()
 SINGLE_PAGE, CONTINUOUS, CONTINUOUS_FACING = range(1,4)
@@ -79,7 +80,9 @@ class Client:
                       StringIO, etc.; if None then the return value is a string
                       containing the PDF.
         """
-        return self.__convert(outstream, 'uri', uri)
+        body = urllib.urlencode(self._prepare_fields(dict(src=uri)))
+        content_type = 'application/x-www-form-urlencoded'
+        return self._post(body, content_type, 'pdf/convert/uri/', outstream)
 
     def convertHtml(self, html, outstream=None):
         """Converts an in-memory html document.
@@ -89,7 +92,11 @@ class Client:
                       StringIO, etc.; if None then the return value is a string
                       containing the PDF.
         """
-        return self.__convert(outstream, 'html', html)
+        if type(html) == unicode:
+            html = html.encode('utf-8')
+        body = urllib.urlencode(self._prepare_fields(dict(src=html)))
+        content_type = 'application/x-www-form-urlencoded'
+        return self._post(body, content_type, 'pdf/convert/html/', outstream)
 
     def convertFile(self, fpath, outstream=None):
         """Converts an html file.
@@ -99,13 +106,14 @@ class Client:
                       StringIO, etc.; if None then the return value is a string
                       containing the PDF.
         """
-        return self.__post_multipart(fpath, outstream)
+        body, content_type = self._encode_multipart_post_data(fpath)
+        return self._post(body, content_type, 'pdf/convert/html/', outstream)
 
     def numTokens(self):
         """Returns the number of available conversion tokens."""
-        uri = self.api_uri + 'user/%s/tokens/' % self.fields['username']
-        response = self.__call_api(uri)
-        return int(response)
+        body = urllib.urlencode(self._prepare_fields())
+        content_type = 'application/x-www-form-urlencoded'
+        return int(self._post(body, content_type, 'user/%s/tokens/' % self.fields['username']))
 
     def useSSL(self, use_ssl):
         if use_ssl:
@@ -131,10 +139,16 @@ class Client:
         self.fields['height'] = value
 
     def setHorizontalMargin(self, value):
-        self.fields['hmargin'] = str(value)
+        self.fields['margin_right'] = self.fields['margin_left'] = str(value)
 
     def setVerticalMargin(self, value):
-        self.fields['vmargin'] = str(value)
+        self.fields['margin_top'] = self.fields['margin_bottom'] = str(value)
+
+    def setPageMargins(self, top, right, bottom, left):
+        self.fields['margin_top'] = str(top)
+        self.fields['margin_right'] = str(right)
+        self.fields['margin_bottom'] = str(bottom)
+        self.fields['margin_left'] = str(left)
 
     def setEncrypted(self, val=True):
         self.fields['encrypted'] = val
@@ -250,47 +264,23 @@ class Client:
     #                       Private stuff
     # 
 
-    def __convert(self, out_stream, method, src):
-        uri = self.api_uri + 'pdf/convert/%s/' % method
-        return self.__call_api(uri, out_stream, src)
-
-    def __call_api(self, uri, out_stream=None, src=None):
-        data = self.__encode_post_data({'src': src})
-        try:
-            obj = urllib2.urlopen(uri, data)
-            if out_stream:
-                while True:
-                    data = obj.read(16384)
-                    if data:
-                        out_stream.write(data)
-                    else:
-                        break
-                return out_stream
-            else:
-                return obj.read()
-        except urllib2.HTTPError, why:
-            raise Error(why.read(), why.code)
-        except urllib2.URLError, why:
-            raise Error(why.reason[1])
-
-    def __encode_post_data(self, extra_data={}):
+    def _prepare_fields(self, extra_data={}):
         result = extra_data.copy()
         for key, val in self.fields.iteritems():
             if val:
                 if type(val) == float:
                     val = str(val).replace(',', '.')
                 result[key] = val
-        return urllib.urlencode(result)
+        return result
 
-    def __encode_multipart_post_data(self, filename):
+    def _encode_multipart_post_data(self, filename):
         boundary = '----------ThIs_Is_tHe_bOUnDary_$'
         body = []
-        for field, value in self.fields.iteritems():
-            if value:
-                body.append('--' + boundary)
-                body.append('Content-Disposition: form-data; name="%s"' % field)
-                body.append('')
-                body.append(str(value))
+        for field, value in self._prepare_fields().iteritems():
+            body.append('--' + boundary)
+            body.append('Content-Disposition: form-data; name="%s"' % field)
+            body.append('')
+            body.append(str(value))
         # filename
         body.append('--' + boundary)
         body.append('Content-Disposition: form-data; name="src"; filename="%s"' % filename)
@@ -303,13 +293,13 @@ class Client:
         body.append('')
         body = '\r\n'.join(body)
         content_type = 'multipart/form-data; boundary=%s' % boundary
-        return content_type, body
+        return body, content_type
 
-    def __post_multipart(self, fpath, out_stream):
+    # sends a POST to the API
+    def _post(self, body, content_type, api_path, outstream=None):
         try:
-            content_type, body = self.__encode_multipart_post_data(fpath)
             conn = self.conn_type(self.host, self.port)
-            conn.putrequest('POST', API_SELECTOR_BASE+'pdf/convert/html/')
+            conn.putrequest('POST', API_SELECTOR_BASE + api_path)
             conn.putheader('content-type', content_type)
             conn.putheader('content-length', str(len(body)))
             conn.endheaders()
@@ -317,124 +307,26 @@ class Client:
             response = conn.getresponse()
             if response.status != 200:
                 raise Error(response.read(), response.status)
-            if out_stream:
+            if outstream:
                 while True:
                     data = response.read(16384)
                     if data:
-                        out_stream.write(data)
+                        outstream.write(data)
                     else:
                         break
-                return out_stream
+                return outstream
             else:
                 return response.read()
+        except httplib.HTTPException, err:
+            raise Error(str(err))
         except socket.gaierror, err:
             raise Error(err[1])
 
 
-
 API_SELECTOR_BASE = '/api/'
 HOST = 'pdfcrowd.com'
 HTTP_PORT = 80
 HTTPS_PORT = 443
 
 
-# ---------------------------------------------------------------------------
-#
-#                                Test
-#
 
-if __name__ == "__main__":
-    import sys
-    import os
-
-    if len(sys.argv) < 2:
-        print "usage: python pdfcrowd.py username apikey [apihost [http-port https-port]]"
-        sys.exit(1)
-
-    if len(sys.argv) > 3:
-        HOST = sys.argv[3]
-
-    if len(sys.argv) == 6:
-        HTTP_PORT = int(sys.argv[4])
-        HTTPS_PORT = int(sys.argv[5])
-
-    print "using %s ports %d %d" % (HOST, HTTP_PORT, HTTPS_PORT)
-
-    os.chdir(os.path.dirname(sys.argv[0]))
-    test_dir = '../test_files'
-    if not os.path.exists(test_dir + '/out'):
-        os.makedirs(test_dir + '/out')
-
-    def out_stream(name, use_ssl):
-        fname = test_dir + '/out/py_client_%s' % name
-        if use_ssl:
-            fname = fname + '_ssl'
-        return open(fname + '.pdf', 'wb')
-
-    html="<html><body>Uploaded content!</body></html>"
-    client = Client(sys.argv[1], sys.argv[2])
-    for use_ssl in [False, True]:
-        client.useSSL(use_ssl)
-        try:
-            ntokens = client.numTokens()
-            client.convertURI('http://www.jagpdf.org/', out_stream('uri', use_ssl))
-            client.convertHtml(html, out_stream('content', use_ssl))
-            client.convertFile(test_dir + '/in/simple.html', out_stream('upload', use_ssl))
-            client.convertFile(test_dir + '/in/archive.tar.gz', out_stream('archive', use_ssl))
-            after_tokens = client.numTokens()
-            print 'remaining tokens:', after_tokens
-            assert ntokens-4 == after_tokens
-        except Error, why:
-            print 'FAILED:', why
-            sys.exit(1)
-    # test individual methods
-    tests = (('setPageWidth', 500),
-             ('setPageHeight', -1),
-             ('setHorizontalMargin', 72),
-             ('setVerticalMargin', 72),
-             ('setEncrypted', True),
-             ('setUserPassword', 'userpwd'),
-             ('setOwnerPassword', 'ownerpwd'),
-             ('setNoPrint', True),
-             ('setNoModify', True),
-             ('setNoCopy', True),
-             ('setPageLayout', CONTINUOUS),
-             ('setPageMode', FULLSCREEN),
-             ('setFooterText', '%p/%n | source %u'),
-             ('enableImages', False),
-             ('enableBackgrounds', False),
-             ('setHtmlZoom', 300),
-             ('enableJavaScript', False),
-             ('enableHyperlinks', False),
-             ('setDefaultTextEncoding', 'iso-8859-1'),
-             ('usePrintMedia', True),
-             ('setMaxPages', 1),
-             ('enablePdfcrowdLogo', True),
-             ('setInitialPdfZoomType', FIT_PAGE),
-             ('setInitialPdfExactZoom', 113),
-             ('setPdfScalingFactor', .5),
-             ('setFooterHtml', '<b>bold</b> and <i>italic</i> <img src="http://pdfcrowd.com/static/images/logo175x30.png" />'),
-             ('setFooterUrl', 'http://google.com'),
-             ('setHeaderHtml', 'page %p out of %n'),
-             ('setHeaderUrl', 'http://google.com'),             
-             ('setAuthor', 'Your Name'),
-             ('setPageBackgroundColor', 'ee82EE'),
-             ('setTransparentBackground', True))
-    try:
-        for method, arg in tests:
-            client = Client(sys.argv[1], sys.argv[2])
-            client.setVerticalMargin("1in")
-            getattr(client, method)(arg)
-            client.convertFile(test_dir + '/in/simple.html', out_stream(method.lower(), False))
-    except Error, why:
-        print 'FAILED', why
-        sys.exit(1)
-    # expected failures
-    client = Client(sys.argv[1], sys.argv[2])
-    try:
-        client.setFailOnNon200(True)
-        client.convertURI("http://pdfcrowd.com/this/url/does/not/exist/")
-        print "FAILED expected an exception"
-        sys.exit(1)
-    except Error, why:
-        pass # expected
